@@ -14,13 +14,34 @@
 #    under the License.
 
 import copy
-
+import io
 import mock
+
 from six.moves import http_client
 
 from sushy import exceptions
 from sushy.resources import base as resource_base
 from sushy.tests.unit import base
+import zipfile
+
+
+BASE_RESOURCE_JSON = {
+    "@odata.type": "#FauxResource.v1_0_0.FauxResource",
+    "Id": "1111AAAA",
+    "Name": "Faux Resource",
+    "@odata.id": "/redfish/v1/FauxResource/1111AAAA",
+    "Oem": {
+        "Contoso": {
+            "@odata.type": "http://contoso.com/schemas/extensions.v1_2_1#contoso.AnvilTypes1",  # noqa
+            "slogan": "Contoso never fail",
+            "disclaimer": "* Most of the time"
+        },
+        "EID_412_ASB_123": {
+            "@odata.type": "http://AnotherStandardsBody/schemas.v1_0_1#styleInfoExt",  # noqa
+            "Style": "Executive"
+        }
+    }
+}
 
 
 class BaseResource(resource_base.ResourceBase):
@@ -29,14 +50,22 @@ class BaseResource(resource_base.ResourceBase):
         pass
 
 
+class BaseResource2(resource_base.ResourceBase):
+    pass
+
+
 class ResourceBaseTestCase(base.TestCase):
 
     def setUp(self):
         super(ResourceBaseTestCase, self).setUp()
         self.conn = mock.Mock()
+        self.conn.get.return_value.json.return_value = (
+            copy.deepcopy(BASE_RESOURCE_JSON))
         self.base_resource = BaseResource(connector=self.conn, path='/Foo',
                                           redfish_version='1.0.2')
         self.assertFalse(self.base_resource._is_stale)
+        self.base_resource2 = BaseResource2(connector=self.conn, path='/Foo',
+                                            redfish_version='1.0.2')
         # refresh() is called in the constructor
         self.conn.reset_mock()
 
@@ -59,6 +88,52 @@ class ResourceBaseTestCase(base.TestCase):
         self.base_resource.invalidate(force_refresh=True)
         self.conn.get.assert_called_once_with(path='/Foo')
 
+    def test_refresh_archive(self):
+        mock_response = mock.Mock(
+            headers={'content-type': 'application/zip'})
+        with open('sushy/tests/unit/json_samples/TestRegistry.zip', 'rb') as f:
+            mock_response.content = f.read()
+        self.conn.get.return_value = mock_response
+
+        resource = BaseResource(connector=self.conn,
+                                path='/Foo',
+                                redfish_version='1.0.2',
+                                reader=resource_base.
+                                JsonArchiveReader('Test.2.0.json'))
+
+        self.assertIsNotNone(resource._json)
+        self.assertEqual('Test.2.0.0', resource._json['Id'])
+
+    @mock.patch.object(resource_base, 'LOG', autospec=True)
+    def test_refresh_archive_not_implemented(self, mock_log):
+        mock_response = mock.Mock(
+            headers={'content-type': 'application/gzip'})
+        self.conn.get.return_value = mock_response
+        BaseResource(connector=self.conn,
+                     path='/Foo',
+                     redfish_version='1.0.2',
+                     reader=resource_base.JsonArchiveReader('Test.2.0.json'))
+        mock_log.error.assert_called_once()
+
+    @mock.patch.object(io, 'BytesIO', autospec=True)
+    def test_refresh_archive_badzip_error(self, mock_io):
+        mock_response = mock.Mock(
+            headers={'content-type': 'application/zip'})
+        mock_io.side_effect = zipfile.BadZipfile('Something wrong')
+        self.conn.get.return_value = mock_response
+
+        self.assertRaises(exceptions.SushyError,
+                          BaseResource, connector=self.conn,
+                          path='/Foo',
+                          redfish_version='1.0.2',
+                          reader=resource_base.
+                          JsonArchiveReader('Test.2.0.json'))
+
+    def test__parse_attributes(self):
+        for oem_vendor in self.base_resource2.oem_vendors:
+            self.assertTrue(oem_vendor in ('Contoso', 'EID_412_ASB_123'))
+        self.assertEqual('base_resource2', self.base_resource2.resource_name)
+
 
 class TestResource(resource_base.ResourceBase):
     """A concrete Test Resource to test against"""
